from functools import wraps
from subprocess import run, PIPE
from typing import Callable, Any

import parse


def qtile_func(func: Callable[..., Any]) -> Callable[..., Any]:
    """
    This decorator makes a function suitable to be used as a parameter for
    `lazy.function()` by making the wrapped function return a function that
    takes the qtile instance as first parameter.
    """
    @wraps(func)
    def wrapper(*args, **kwargs):
        def f(qtile):
            func(*args, **kwargs)
        return f
    return wrapper


def notify(summary: str, body: str, urgency: str='normal') -> None:
    run(['notify-send', '-u', urgency, summary, body])
